@@ -25,8 +25,8 @@
 #include "config.h"
 #include "dh-book-tree.h"
 #include <glib/gi18n-lib.h>
-#include "dh-book-manager.h"
 #include "dh-book.h"
+#include "dh-book-list.h"
 #include "dh-settings.h"
 
 /**
@@ -46,13 +46,6 @@
  * emitted. Only one element can be selected at a time.
  */
 
-typedef struct {
-        const gchar *uri;
-        GtkTreeIter  iter;
-        GtkTreePath *path;
-        guint        found : 1;
-} FindURIData;
-
 typedef struct {
         DhProfile *profile;
         GtkTreeStore *store;
@@ -60,6 +53,13 @@ typedef struct {
         GtkMenu *context_menu;
 } DhBookTreePrivate;
 
+typedef struct {
+        const gchar *uri;
+        GtkTreeIter iter;
+        GtkTreePath *path;
+        guint found : 1;
+} FindURIData;
+
 enum {
         LINK_SELECTED,
         N_SIGNALS
@@ -100,26 +100,19 @@ book_tree_selection_changed_cb (GtkTreeSelection *selection,
                                 DhBookTree       *tree)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        GtkTreeIter     iter;
+        DhLink *link;
 
-        if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
-                DhLink *link;
+        link = dh_book_tree_get_selected_link (tree);
 
-                gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
-                                    &iter,
-                                    COL_LINK, &link,
-                                    -1);
-
-                if (link != NULL &&
-                    link != priv->selected_link) {
-                        g_clear_pointer (&priv->selected_link, (GDestroyNotify)dh_link_unref);
-                        priv->selected_link = dh_link_ref (link);
-                        g_signal_emit (tree, signals[LINK_SELECTED], 0, link);
-                }
-
-                if (link != NULL)
-                        dh_link_unref (link);
+        if (link != NULL &&
+            link != priv->selected_link) {
+                g_clear_pointer (&priv->selected_link, (GDestroyNotify)dh_link_unref);
+                priv->selected_link = dh_link_ref (link);
+                g_signal_emit (tree, signals[LINK_SELECTED], 0, link);
         }
+
+        if (link != NULL)
+                dh_link_unref (link);
 }
 
 static void
@@ -143,9 +136,8 @@ book_tree_setup_selection (DhBookTree *tree)
  *  - Or the language group which should be just after our given language group.
  *  - Or both.
  *
- * FIXME: not great code. Maybe add a new column in the GtkTreeModel storing a
- * DhLanguage object. Instead of @language as a string, it would be a
- * DhLanguage.
+ * FIXME: not great code. Maybe have a DhLanguage object, and add a new column
+ * in the GtkTreeModel storing a DhLanguage object instead of a string.
  */
 static void
 book_tree_find_language_group (DhBookTree  *tree,
@@ -236,7 +228,7 @@ book_tree_find_book (DhBookTree        *tree,
                      gboolean          *next_found)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        GtkTreeIter     loop_iter;
+        GtkTreeIter loop_iter;
 
         g_assert ((exact_iter != NULL && exact_found != NULL) ||
                   (next_iter != NULL && next_found != NULL));
@@ -301,18 +293,17 @@ book_tree_insert_node (DhBookTree  *tree,
 
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        DhLink         *link;
-        PangoWeight     weight;
-        GNode          *child;
+        DhLink *link;
+        PangoWeight weight;
+        GNode *child;
 
         link = node->data;
         g_assert (link != NULL);
 
-        if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
+        if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK)
                 weight = PANGO_WEIGHT_BOLD;
-        } else {
+        else
                 weight = PANGO_WEIGHT_NORMAL;
-        }
 
         gtk_tree_store_set (priv->store,
                             current_iter,
@@ -343,12 +334,12 @@ book_tree_add_book_to_store (DhBookTree *tree,
 
         /* If grouping by language we need to add the language categories */
         if (get_group_books_by_language (tree)) {
-                GtkTreeIter  language_iter;
-                gboolean     language_iter_found;
-                GtkTreeIter  next_language_iter;
-                gboolean     next_language_iter_found;
+                GtkTreeIter language_iter;
+                gboolean language_iter_found;
+                GtkTreeIter next_language_iter;
+                gboolean next_language_iter_found;
                 const gchar *language_title;
-                gboolean     new_language = FALSE;
+                gboolean new_language = FALSE;
 
                 language_title = dh_book_get_language (book);
 
@@ -374,11 +365,11 @@ book_tree_add_book_to_store (DhBookTree *tree,
 
                         gtk_tree_store_set (priv->store,
                                             &language_iter,
-                                            COL_TITLE,      language_title,
-                                            COL_LINK,       NULL,
-                                            COL_BOOK,       NULL,
-                                            COL_WEIGHT,     PANGO_WEIGHT_BOLD,
-                                            COL_UNDERLINE,  PANGO_UNDERLINE_SINGLE,
+                                            COL_TITLE, language_title,
+                                            COL_LINK, NULL,
+                                            COL_BOOK, NULL,
+                                            COL_WEIGHT, PANGO_WEIGHT_BOLD,
+                                            COL_UNDERLINE, PANGO_UNDERLINE_SINGLE,
                                             -1);
 
                         new_language = TRUE;
@@ -402,7 +393,7 @@ book_tree_add_book_to_store (DhBookTree *tree,
                 } else {
                         GtkTreeIter first_book_iter;
                         GtkTreeIter next_book_iter;
-                        gboolean    next_book_iter_found;
+                        gboolean next_book_iter_found;
 
                         /* The language will have at least one book, so we move iter to it */
                         gtk_tree_model_iter_children (GTK_TREE_MODEL (priv->store),
@@ -432,7 +423,7 @@ book_tree_add_book_to_store (DhBookTree *tree,
         } else {
                 /* No language grouping, just order by book title */
                 GtkTreeIter next_book_iter;
-                gboolean    next_book_iter_found;
+                gboolean next_book_iter_found;
 
                 book_tree_find_book (tree,
                                      book,
@@ -463,26 +454,23 @@ book_tree_add_book_to_store (DhBookTree *tree,
 }
 
 static void
-book_tree_book_created_or_enabled_cb (DhBookManager *book_manager,
-                                      DhBook        *book,
-                                      DhBookTree    *tree)
+add_book_cb (DhBookList *book_list,
+             DhBook     *book,
+             DhBookTree *tree)
 {
-        if (!dh_book_get_enabled (book))
-                return;
-
         book_tree_add_book_to_store (tree, book);
 }
 
 static void
-book_tree_book_deleted_or_disabled_cb (DhBookManager *book_manager,
-                                       DhBook        *book,
-                                       DhBookTree    *tree)
+remove_book_cb (DhBookList *book_list,
+                DhBook     *book,
+                DhBookTree *tree)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        GtkTreeIter     exact_iter;
-        gboolean        exact_iter_found = FALSE;
-        GtkTreeIter     language_iter;
-        gboolean        language_iter_found = FALSE;
+        GtkTreeIter exact_iter;
+        gboolean exact_iter_found = FALSE;
+        GtkTreeIter language_iter;
+        gboolean language_iter_found = FALSE;
 
         if (get_group_books_by_language (tree)) {
                 GtkTreeIter first_book_iter;
@@ -537,19 +525,30 @@ book_tree_book_deleted_or_disabled_cb (DhBookManager *book_manager,
 static void
 book_tree_init_selection (DhBookTree *tree)
 {
-        DhBookTreePrivate   *priv;
+        DhBookTreePrivate *priv;
         GtkTreeSelection *selection;
-        GtkTreeIter       iter;
-        gboolean          iter_found = FALSE;
+        GtkTreeIter iter;
+        gboolean iter_found = FALSE;
 
         priv = dh_book_tree_get_instance_private (tree);
 
         /* Mark the first item as selected, or it would get automatically
-         * selected when the treeview will get focus; but that's not even
-         * enough as a selection changed would still be emitted when there
-         * is no change, hence the manual tracking of selection in
-         * selected_link.
-         *   https://bugzilla.gnome.org/show_bug.cgi?id=492206
+         * selected when the treeview will get focus (a behavior that we want to
+         * avoid); but that's not even enough as a selection ::changed would
+         * still be emitted when there is no change, hence the manual tracking
+         * of selection with priv->selected_link.
+         *
+         * If there is no manual tracking with selected_link, there is this bug:
+         * 1. Open Devhelp.
+         * 2. The first book is initially selected (thanks to this function),
+         *    OK.
+         * 3. Click on the arrow of another book to expand it.
+         * --> The other book gets correctly expanded (and not selected), but
+         *     the selection ::changed signal is emitted for the *first* book
+         *     (even though it was already selected, strange).
+         *
+         * https://bugzilla.gnome.org/show_bug.cgi?id=492206 - GtkTreeView bug
+         * https://bugzilla.gnome.org/show_bug.cgi?id=603040 - Devhelp bug
          */
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
         g_signal_handlers_block_by_func (selection,
@@ -579,12 +578,12 @@ book_tree_init_selection (DhBookTree *tree)
                                     COL_LINK, &link,
                                     -1);
 
+                if (link == NULL || dh_link_get_link_type (link) != DH_LINK_TYPE_BOOK)
+                        g_warn_if_reached ();
+
                 g_clear_pointer (&priv->selected_link, (GDestroyNotify)dh_link_unref);
                 priv->selected_link = link;
                 gtk_tree_selection_select_iter (selection, &iter);
-
-                if (dh_link_get_link_type (link) != DH_LINK_TYPE_BOOK)
-                        g_warn_if_reached ();
         }
 
         g_signal_handlers_unblock_by_func (selection,
@@ -596,7 +595,7 @@ static void
 book_tree_populate_tree (DhBookTree *tree)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        DhBookManager *book_manager;
+        GList *books;
         GList *l;
 
         gtk_tree_view_set_model (GTK_TREE_VIEW (tree), NULL);
@@ -604,16 +603,11 @@ book_tree_populate_tree (DhBookTree *tree)
         gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
                                  GTK_TREE_MODEL (priv->store));
 
-        /* This list comes in order, but we don't really mind */
-        book_manager = dh_book_manager_get_singleton ();
-        for (l = dh_book_manager_get_books (book_manager);
-             l != NULL;
-             l = l->next) {
-                DhBook *book = DH_BOOK (l->data);
+        books = dh_book_list_get_books (dh_profile_get_book_list (priv->profile));
 
-                /* Only add enabled books to the tree */
-                if (dh_book_get_enabled (book))
-                        book_tree_add_book_to_store (tree, book);
+        for (l = books; l != NULL; l = l->next) {
+                DhBook *book = DH_BOOK (l->data);
+                book_tree_add_book_to_store (tree, book);
         }
 
         book_tree_init_selection (tree);
@@ -682,7 +676,7 @@ dh_book_tree_constructed (GObject *object)
 {
         DhBookTree *tree = DH_BOOK_TREE (object);
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        DhBookManager *book_manager;
+        DhBookList *book_list;
         DhSettings *settings;
 
         if (G_OBJECT_CLASS (dh_book_tree_parent_class)->constructed != NULL)
@@ -693,31 +687,19 @@ dh_book_tree_constructed (GObject *object)
 
         book_tree_setup_selection (tree);
 
-        book_manager = dh_book_manager_get_singleton ();
+        book_list = dh_profile_get_book_list (priv->profile);
 
-        g_signal_connect_object (book_manager,
-                                 "book-created",
-                                 G_CALLBACK (book_tree_book_created_or_enabled_cb),
+        g_signal_connect_object (book_list,
+                                 "add-book",
+                                 G_CALLBACK (add_book_cb),
                                  tree,
-                                 0);
+                                 G_CONNECT_AFTER);
 
-        g_signal_connect_object (book_manager,
-                                 "book-enabled",
-                                 G_CALLBACK (book_tree_book_created_or_enabled_cb),
+        g_signal_connect_object (book_list,
+                                 "remove-book",
+                                 G_CALLBACK (remove_book_cb),
                                  tree,
-                                 0);
-
-        g_signal_connect_object (book_manager,
-                                 "book-deleted",
-                                 G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
-                                 tree,
-                                 0);
-
-        g_signal_connect_object (book_manager,
-                                 "book-disabled",
-                                 G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
-                                 tree,
-                                 0);
+                                 G_CONNECT_AFTER);
 
         settings = dh_profile_get_settings (priv->profile);
         g_signal_connect_object (settings,
@@ -871,7 +853,7 @@ dh_book_tree_class_init (DhBookTreeClass *klass)
 static void
 book_tree_add_columns (DhBookTree *tree)
 {
-        GtkCellRenderer   *cell;
+        GtkCellRenderer *cell;
         GtkTreeViewColumn *column;
 
         column = gtk_tree_view_column_new ();
@@ -904,7 +886,7 @@ dh_book_tree_init (DhBookTree *tree)
                                           DH_TYPE_BOOK,
                                           PANGO_TYPE_WEIGHT,
                                           PANGO_TYPE_UNDERLINE);
-        priv->selected_link = NULL;
+
         gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
                                  GTK_TREE_MODEL (priv->store));
 
@@ -945,12 +927,43 @@ dh_book_tree_get_profile (DhBookTree *tree)
         return priv->profile;
 }
 
-static gboolean
-book_tree_find_uri_foreach (GtkTreeModel *model,
-                            GtkTreePath  *path,
-                            GtkTreeIter  *iter,
-                            FindURIData  *data)
+/**
+ * dh_book_tree_get_selected_link:
+ * @tree: a #DhBookTree.
+ *
+ * Returns: (transfer full) (nullable): the currently selected #DhLink in @tree,
+ * or %NULL if the selection is empty or if a language group row is selected.
+ * Unref with dh_link_unref() when no longer needed.
+ * Since: 3.30
+ */
+DhLink *
+dh_book_tree_get_selected_link (DhBookTree *tree)
 {
+        GtkTreeSelection *selection;
+        GtkTreeModel *model;
+        GtkTreeIter iter;
+        DhLink *link;
+
+        g_return_val_if_fail (DH_IS_BOOK_TREE (tree), NULL);
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+        if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+                return NULL;
+
+        gtk_tree_model_get (model, &iter,
+                            COL_LINK, &link,
+                            -1);
+
+        return link;
+}
+
+static gboolean
+book_tree_find_uri_foreach_func (GtkTreeModel *model,
+                                 GtkTreePath  *path,
+                                 GtkTreeIter  *iter,
+                                 gpointer      _data)
+{
+        FindURIData *data = _data;
         DhLink *link;
 
         gtk_tree_model_get (model, iter,
@@ -981,22 +994,28 @@ book_tree_find_uri_foreach (GtkTreeModel *model,
  * @tree: a #DhBookTree.
  * @uri: the URI to select.
  *
- * Selects the given @uri.
+ * Selects the row corresponding to @uri. It searches in the tree a #DhLink
+ * being at @uri (if it's an exact match), or containing @uri (if @uri contains
+ * an anchor).
  */
 void
 dh_book_tree_select_uri (DhBookTree  *tree,
                          const gchar *uri)
 {
-        DhBookTreePrivate   *priv = dh_book_tree_get_instance_private (tree);
+        DhBookTreePrivate *priv;
         GtkTreeSelection *selection;
-        FindURIData       data;
-        DhLink           *link;
+        FindURIData data;
+
+        g_return_if_fail (DH_IS_BOOK_TREE (tree));
+        g_return_if_fail (uri != NULL);
+
+        priv = dh_book_tree_get_instance_private (tree);
 
         data.found = FALSE;
         data.uri = uri;
 
         gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
-                                (GtkTreeModelForeachFunc) book_tree_find_uri_foreach,
+                                book_tree_find_uri_foreach_func,
                                 &data);
 
         if (!data.found)
@@ -1004,79 +1023,20 @@ dh_book_tree_select_uri (DhBookTree  *tree,
 
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
 
-        /* Do not re-select (which will expand current additionally) if already
-         * there. */
+        /* Do not re-expand/select/scroll if already there. */
         if (gtk_tree_selection_iter_is_selected (selection, &data.iter))
                 goto out;
 
-        /* FIXME: it's strange to block the signal here. The signal handler
-         * should probably be blocked in DhWindow instead.
+        /* The order is important here: select_iter() doesn't work if the row is
+         * hidden.
          */
-        g_signal_handlers_block_by_func (selection,
-                                         book_tree_selection_changed_cb,
-                                         tree);
-
         gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree), data.path);
-
-        gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
-                            &data.iter,
-                            COL_LINK, &link,
-                            -1);
-        g_clear_pointer (&priv->selected_link, (GDestroyNotify)dh_link_unref);
-        priv->selected_link = link;
         gtk_tree_selection_select_iter (selection, &data.iter);
 
-        gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree), data.path, NULL, 0);
-
-        g_signal_handlers_unblock_by_func (selection,
-                                           book_tree_selection_changed_cb,
-                                           tree);
+        gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree),
+                                      data.path, NULL,
+                                      FALSE, 0.0, 0.0);
 
 out:
         gtk_tree_path_free (data.path);
 }
-
-/**
- * dh_book_tree_get_selected_book:
- * @tree: a #DhBookTree.
- *
- * Returns: (nullable) (transfer full): the #DhLink of the selected book, or
- * %NULL if there is no selection. Unref with dh_link_unref() when no longer
- * needed.
- */
-DhLink *
-dh_book_tree_get_selected_book (DhBookTree *tree)
-{
-        GtkTreeSelection *selection;
-        GtkTreeModel     *model;
-        GtkTreeIter       iter;
-
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-        if (!gtk_tree_selection_get_selected (selection, &model, &iter))
-                return NULL;
-
-        /* Depending on whether books are grouped by language, the book link can
-         * be at a different depth. And it's safer to check that the returned
-         * link has the good type. So walk up the tree to find the book.
-         */
-        while (TRUE) {
-                DhLink *link;
-                GtkTreeIter parent;
-
-                gtk_tree_model_get (model, &iter,
-                                    COL_LINK, &link,
-                                    -1);
-
-                if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK)
-                        return link;
-
-                dh_link_unref (link);
-
-                if (!gtk_tree_model_iter_parent (model, &parent, &iter))
-                        break;
-
-                iter = parent;
-        }
-
-        g_return_val_if_reached (NULL);
-}
