@@ -19,6 +19,7 @@
  * along with Devhelp.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #include "dh-settings.h"
 #include "dh-settings-builder.h"
 
@@ -40,8 +41,8 @@
  * the #DhSettings wrapper API instead. So instead of using g_settings_bind(),
  * you should use g_object_bind_property() with a #DhSettings property as the
  * source. This has the small drawback that the writability of the #GSettings
- * key cannot be bound to the "sensitive" property of the preferences widget
- * (see g_settings_bind()), if this feature is really wanted the #DhSettings API
+ * keys are not exposed, it is assumed that all keys are writable; if knowing
+ * the writability of the #GSettings keys is really wanted, the #DhSettings API
  * can be changed to expose publicly the #GSettings objects (for that purpose
  * only).
  *
@@ -72,8 +73,9 @@
  */
 
 /* libdevhelp GSettings schema IDs */
-#define SETTINGS_SCHEMA_ID_CONTENTS             "org.gnome.libdevhelp-3.contents"
-#define SETTINGS_SCHEMA_ID_FONTS                "org.gnome.libdevhelp-3.fonts"
+#define LIBDEVHELP_GSCHEMA_PREFIX       "org.gnome.libdevhelp-" LIBDEVHELP_API_VERSION
+#define SETTINGS_SCHEMA_ID_CONTENTS     LIBDEVHELP_GSCHEMA_PREFIX ".contents"
+#define SETTINGS_SCHEMA_ID_FONTS        LIBDEVHELP_GSCHEMA_PREFIX ".fonts"
 
 /* Provided by the gsettings-desktop-schemas module. */
 #define SETTINGS_SCHEMA_ID_DESKTOP_INTERFACE    "org.gnome.desktop.interface"
